@@ -2,7 +2,7 @@
 #'
 #'  Distance metric whose unit ball is a given, symmetric, convex polygon.
 #'
-#' $Revision: 1.19 $  $Date: 2022/03/26 03:06:17 $
+#' $Revision: 1.20 $  $Date: 2022/05/21 09:52:11 $
 
 
 convexmetric <- local({
@@ -327,7 +327,7 @@ convexmetric <- local({
         },
         distmap.psp=function(X, ...) {
           warn.unsupported.args(list(extras=TRUE, clip=FALSE), ...)
-          w <- do.call.matched(as.mask.psp, list(x=quote(X), ...),
+          w <- do.call.matched(psp2mask, list(x=quote(X), ...),
                                extrargs=names(formals(as.mask))[-1])
           convexdistmapmask(w, spK)
         },
