@@ -1,7 +1,7 @@
 #
 #   plot.im.R
 #
-#  $Revision: 1.177 $   $Date: 2025/08/19 09:00:53 $
+#  $Revision: 1.178 $   $Date: 2025/12/21 02:18:59 $
 #
 #  Plotting code for pixel images
 #
@@ -248,7 +248,7 @@ plot.im <- local({
                      add=FALSE, clipwin=NULL,
                      col=NULL, reverse.col=FALSE,
                      valuesAreColours=NULL, log=FALSE,
-                     ncolours=256, gamma=1, 
+                     ncolours=256, gamma=1, opacity=1,
                      ribbon=show.all, show.all=!add,
                      drop.ribbon=FALSE,
                      ribside=c("right", "left", "bottom", "top"),
@@ -274,6 +274,12 @@ plot.im <- local({
     col.given <- !is.null(col)
     dotargs <- list(...)
 
+    if(!missing(opacity)) {
+      check.1.real(opacity)
+      stopifnot(opacity >= 0)
+      stopifnot(opacity <= 1)
+    }
+
     stopifnot(is.list(ribargs))
     user.ticks <- ribargs$at
     user.nint <- ribargs$nint
@@ -609,6 +615,11 @@ plot.im <- local({
       colourinfo$col <- to.grey(colourinfo$col)
     }
 
+    if(opacity != 1) {
+      ## make partially transparent colours
+      colourinfo$col <- to.transparent(colourinfo$col, fraction=opacity)
+    }
+
     if(isTRUE(reverse.col) && !valuesAreColours) {
       ## reverse the colour sequence (using rev.colourmap or rev.default)
       colourinfo$col <- rev(colourinfo$col)
@@ -664,7 +675,7 @@ plot.im <- local({
 
     ##  ........ decide whether to use rasterImage .........
 
-    if(!isFALSE(useRaster)) {
+    if(!isFALSE(useRaster) && do.plot) {
       ## get device capabilities
       ##      (this will start a graphics device if none is active)
       rasterable <- safeDevCapabilities()$rasterImage
